:
# NAME:
#	install_funcs.sh - build install list
#
# DESCRIPTION:
#	This module contains the list building logic used by
#	install_files and sync_tree.
#	
#	We search "DIR_LIST" (see config_setup.sh(8)) for file lists
#	(install_files.list).  See install_files(8) for details.
#	
#	Lists:
#
#	Apart from the standard lists install_files.list and
#	install.client.files-list, we search "DIR_LIST" for a file
#	called install_lists.list.  If found it is expected to name a
#	number of install lists which will in turn be searched for via
#	"DIR_LIST".   This allows packages of files to have a separate
#	install list which is only actioned on certain hosts.
#
#	Excludes:
#
#	Often one or more hosts require the same basic configuration
#	but with certain omissions.  For this reason we first search
#	"DIR_LIST" looking for file lists (exclude_files.list) which
#	simply contain pathnames to be skipped.
#.nf
#
#		"exclude"
#.fi
#   
#	Any "path" which matches "exclude" or is in a sub-directory of
#	it will be skipped.  For obvious reasons, we treat any
#	removed_files.list as though it were an exclude_files.list.
#
#	Similarly, any install list named in an exclude_lists.list
#	will be skipped.
#
# SEE ALSO:
#	install_files(8), config_setup.sh(8), remove_files(8), set_perms(8)
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#
#	

# RCSid:
#	$Id: install_funcs.sh,v 1.21 2001/12/10 21:49:40 sjg Exp $
#
#	@(#) Copyright (c) 1994 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

. $Mydir/config_funcs.sh

# this is all we want shpp to do for us.
SHPP_DO=.include,.require
SHPP=shpp
export SHPP

# update_file can tell us to only do certain lists

LIST=${LIST:-`IFS=", "; echo $LIST_ONLY`}
#
# Look for install.client.files-list too for
# compatability with Rodney's config files.
#
LIST=${LIST:-"install_files.$EXT install.client.files-$EXT"}

# first build an exclude list. 
# for each element we skip an exact match as well as
# any sub-dirs.
EXCLUDES="$UPDATE_EXCLUDES `_build_list exclude_files.list no_install.list remove_files.list | sed 's/[		].*//'`"

tf=${TMP:-/tmp}/$Myname.$$
> $tf

BuildLists() {
	for llist in $*
	do
		ilists=`find_all /$llist`
		#
		# sorting ilists can cause havoc
		#
		for list in `cat ${ilists:-/dev/null} | grep -v '^#'`
		do
			skipl=
			for ex in $elists
			do
eval "case '$list' in $ex|$ex.list) skipl=yes; break;; esac"
			done
			#
			# now do like .require
			#
			if [ -s ${INCLUDED_LIST:-/dev/null} ]; then
				case `grep "^$list" $INCLUDED_LIST` in
				"") ;;
				*) continue;;
				esac
			fi
			[ "$skipl" ] || build_list $list
		done
	done >> $tf
}

if [ "x$LIST_ONLY" = x ]; then
	elists=`_build_list exclude_lists.list no_install_lists.list`
	#
	# pre_install_lists.list allows us to do things that must be
	# done first - eg like setting up and mounting loopback fs
	# see share.list as an example of something we might want done
	# at this point.
	# 
	BuildLists pre_install_lists.list install_lists.list
fi

# we used to do this first, now its almost last.
build_list $LIST >> $tf

if [ "x$LIST_ONLY" = x ]; then
	# do this one last
	BuildLists post_install_lists.list
fi

#
# when building a chroot env we need to clone certain devices.
# this thing does an ls -lL /dev/foo and extracts the type,
# major and minor numbers from the output.
#
# usage:
#	.eval CloneDev /chroot 666 /dev/zero /dev/tcp /dev/udp
#
# the mode can be omitted or changed between devs.
# if /dev/foo does not exist, then /chroot/dev/foo will be skipped.
# if /chroot/dev/foo exists it will be skipped.
#
CloneDev() {
	_dest=$DESTDIR$1
	shift
	_mode=666

	for d in $*
	do
		case "$d" in
		[0-9]*) _mode=$d; continue;;
		esac
		exists $_dest$d && continue
		/bin/ls -lL $DESTDIR$d 2>/dev/null |
		sed -n "s:^\(.\).*[^0-9]\([0-9][0-9]*\),[^0-9]*\([0-9][0-9]*\)[^0-9][^/]*\(/.*\)\$:${MKNOD:-mknod} $_dest$d \1 \2 \3 \&\& ${CHMOD:-chmod} $_mode $_dest$d:p"
	done | ${_shell:-sh} -x
}
